# 2019-May
# github.com/ludlows
# Python Wrapper for PESQ Score (narrow band and wide band)

import numpy as np
from .cypesq import cypesq

USAGE = """
       Run model on reference ref and degraded deg
       Sample rate (fs) - No default. Must select either 8000 or 16000.
       Note there is narrow band (nb) mode only when sampling rate is 8000Hz.
       """
def pesq(fs, ref, deg, mode):
    """
    Args:
        ref: numpy 1D array, reference audio signal 
        deg: numpy 1D array, degraded audio signal
        fs:  integer, sampling rate
        mode: 'wb' (wide-band) or 'nb' (narrow-band)
    Returns:
        pesq_score: float, P.862.2 Prediction (MOS-LQO)
    """
    if mode != 'wb' and mode != 'nb':
        print(USAGE)
        raise ValueError("mode should be either 'nb' or 'wb'")
    if fs != 8000 and fs != 16000:
        print(USAGE)
        raise ValueError("fs (sampling frequency) should be either 8000 or 16000")
    if fs == 8000 and mode == 'wb':
        print(USAGE)
        raise ValueError("no wide band mode if fs = 8000")
    maxval = max(np.max(np.abs(ref/1.0)), np.max(np.abs(deg/1.0)))
    if mode == 'wb':
        return cypesq(fs, (ref/maxval).astype(np.float32), (deg/maxval).astype(np.float32), 1)
    return cypesq(fs, (ref/maxval).astype(np.float32), (deg/maxval).astype(np.float32), 0)




# distutils: language=C

#2019-May
#github.com/ludlows
#Python Wrapper for PESQ Score (narrow band and wide band)

import cython
cimport numpy as np

cdef extern from "pesq.h":
    DEF MAXNUTTERANCES = 50
    DEF NB_MODE = 0
    DEF WB_MODE = 1

    ctypedef struct SIGNAL_INFO:
        char  path_name[512]
        char  file_name[128]
        long  Nsamples
        long  apply_swap
        long  input_filter

        float * data
        float * VAD
        float * logVAD
    
    ctypedef struct ERROR_INFO:
        long Nutterances
        long Largest_uttsize
        long Nsurf_samples

        long  Crude_DelayEst
        float Crude_DelayConf
        long  UttSearch_Start[MAXNUTTERANCES]
        long  UttSearch_End[MAXNUTTERANCES]
        long  Utt_DelayEst[MAXNUTTERANCES]
        long  Utt_Delay[MAXNUTTERANCES]
        float Utt_DelayConf[MAXNUTTERANCES]
        long  Utt_Start[MAXNUTTERANCES]
        long  Utt_End[MAXNUTTERANCES]

        float pesq_mos
        float mapped_mos
        short mode

cdef extern from "pesqio.h":
    cdef void select_rate(long sample_rate, long * Error_Flag, char ** Error_Type)


cdef extern from "pesqmain.h":
    cdef void pesq_measure(SIGNAL_INFO * ref_info, SIGNAL_INFO * deg_info, ERROR_INFO * err_info, long * Error_Flag, char ** Error_Type)


cpdef object cypesq(long sample_rate, np.ndarray[float, ndim=1, mode="c"] ref_data,  np.ndarray[float, ndim=1, mode="c"] deg_data, int mode):
    # select rate
    cdef long error_flag = 0;
    cdef char * error_type = "unknown";
    select_rate(sample_rate, &error_flag, &error_type)
    if error_flag != 0:
        return -1
    # assign signal
    cdef long length_ref
    cdef long length_deg
    length_ref = ref_data.shape[0]
    length_deg = deg_data.shape[0]

    cdef SIGNAL_INFO ref_info, deg_info

    cdef char*  ref_name = "reference-signal"
    cdef char*  deg_name = "degrade-signal"
    
    cdef int i = 0;
    while ref_name[i]: 
        ref_info.path_name[i] = ref_name[i]
        ref_info.file_name[i] = ref_name[i]
        i += 1
    
    ref_info.path_name[i] = 0
    ref_info.file_name[i] = 0

    i = 0
    while deg_name[i]:
        deg_info.path_name[i] = deg_name[i]
        deg_info.file_name[i] = deg_name[i]
        i += 1
    
    deg_info.path_name[i] = 0
    deg_info.file_name[i] = 0


    ref_info.Nsamples = length_ref
    ref_info.apply_swap = 0
    ref_info.input_filter = 1
    ref_info.data = &(ref_data[0])


    deg_info.Nsamples = length_deg
    deg_info.apply_swap = 0
    deg_info.input_filter = 1
    deg_info.data = &(deg_data[0])

    # assign error info
    cdef ERROR_INFO err_info
    err_info.mode = NB_MODE
    if mode == 1:
        ref_info.input_filter = 2
        deg_info.input_filter = 2
        err_info.mode = WB_MODE

    pesq_measure(&ref_info, &deg_info, &err_info, &error_flag, &error_type);
    if error_flag!=0:
        return -1
    return err_info.mapped_mos





# 2019-May
# github.com/ludlows
# Python Wrapper for PESQ Score (narrow band and wide band)
from setuptools import find_packages
from setuptools import setup, Extension


with open("README.md", "r") as fh:
    long_description = fh.read()


class CyPesqExtension(Extension):
    def __init__(self, *args, **kwargs):
        self._include = []
        super().__init__(*args, **kwargs)

    @property
    def include_dirs(self):
        import numpy
        return self._include + [numpy.get_include()]

    @include_dirs.setter
    def include_dirs(self, dirs):
        self._include = dirs


extensions = [
    CyPesqExtension(
        "cypesq",
        ["pesq/cypesq.pyx", "pesq/dsp.c", "pesq/pesqdsp.c","pesq/pesqmod.c"],
        include_dirs=['pesq'],
        language="c")
]
setup(
    name="pesq",
    version="0.0.2",
    author="ludlows",
    description="Python Wrapper for PESQ Score (narrow band and wide band)",
    long_description=long_description,
    long_description_content_type="text/markdown",
    url="https://github.com/ludlows/python-pesq",
    packages=find_packages(),
    package_data={'pesq':["*.pyx", "*.h", "dsp.c", "pesqdsp.c", "pesqmod.c"]},
    ext_package='pesq',
    ext_modules=extensions,
    setup_requires=['setuptools>=18.0', 'cython', 'numpy', 'pytest-runner'],
    tests_require=['pytest'],
    classifiers=[
        "Programming Language :: Python",
        "License :: OSI Approved :: MIT License",
        "Operating System :: OS Independent",
    ]
)
